@@ -1,6 +1,8 @@
 const t = require('tap')
 const requireInject = require('require-inject')
 const { resolve, delimiter } = require('path')
+const OUTPUT = []
+const output = (...msg) => OUTPUT.push(msg)
 
 const ARB_CTOR = []
 const ARB_ACTUAL_TREE = {}
@@ -29,6 +31,7 @@ const npm = {
     call: '',
     package: [],
     legacyPeerDeps: false,
+    shell: 'shell-cmd',
   },
   localPrefix: 'local-prefix',
   localBin: 'local-bin',
@@ -91,6 +94,7 @@ const mocks = {
   pacote,
   read,
   'mkdirp-infer-owner': mkdirp,
+  '../../lib/utils/output.js': output,
 }
 const exec = requireInject('../../lib/exec.js', mocks)
 
@@ -123,7 +127,7 @@ t.test('npx foo, bin already exists locally', async t => {
   await exec(['foo'], er => {
     t.ifError(er, 'npm exec')
   })
-  t.strictSame(RUN_SCRIPTS, [{
+  t.match(RUN_SCRIPTS, [{
     pkg: { scripts: { npx: 'foo' }},
     banner: false,
     path: process.cwd(),
@@ -147,7 +151,7 @@ t.test('npx foo, bin already exists globally', async t => {
   await exec(['foo'], er => {
     t.ifError(er, 'npm exec')
   })
-  t.strictSame(RUN_SCRIPTS, [{
+  t.match(RUN_SCRIPTS, [{
     pkg: { scripts: { npx: 'foo' }},
     banner: false,
     path: process.cwd(),
@@ -193,6 +197,72 @@ t.test('npm exec foo, already present locally', async t => {
   }])
 })
 
+t.test('npm exec <noargs>, run interactive shell', async t => {
+  CI_NAME = null
+  const { isTTY } = process.stdin
+  process.stdin.isTTY = true
+  t.teardown(() => process.stdin.isTTY = isTTY)
+
+  const run = async (t, doRun = true) => {
+    LOG_WARN.length = 0
+    ARB_CTOR.length = 0
+    MKDIRPS.length = 0
+    ARB_REIFY.length = 0
+    OUTPUT.length = 0
+    await exec([], er => {
+      if (er)
+        throw er
+    })
+    t.strictSame(MKDIRPS, [], 'no need to make any dirs')
+    t.strictSame(ARB_CTOR, [], 'no need to instantiate arborist')
+    t.strictSame(ARB_REIFY, [], 'no need to reify anything')
+    t.equal(PROGRESS_ENABLED, true, 'progress re-enabled')
+    if (doRun) {
+      t.match(RUN_SCRIPTS, [{
+        pkg: { scripts: { npx: 'shell-cmd' } },
+        banner: false,
+        path: process.cwd(),
+        stdioString: true,
+        event: 'npx',
+        env: { PATH: process.env.PATH },
+        stdio: 'inherit',
+      }])
+    } else
+      t.strictSame(RUN_SCRIPTS, [])
+    RUN_SCRIPTS.length = 0
+  }
+
+  t.test('print message when tty and not in CI', async t => {
+    CI_NAME = null
+    process.stdin.isTTY = true
+    await run(t)
+    t.strictSame(LOG_WARN, [])
+    t.strictSame(OUTPUT, [
+      ['\nEntering npm script environment\nType \'exit\' or ^D when finished\n'],
+    ], 'printed message about interactive shell')
+  })
+
+  t.test('no message when not TTY', async t => {
+    CI_NAME = null
+    process.stdin.isTTY = false
+    await run(t)
+    t.strictSame(LOG_WARN, [])
+    t.strictSame(OUTPUT, [], 'no message about interactive shell')
+  })
+
+  t.test('print warning when in CI and interactive', async t => {
+    CI_NAME = 'travis-ci'
+    process.stdin.isTTY = true
+    await run(t, false)
+    t.strictSame(LOG_WARN, [
+      ['exec', 'Interactive mode disabled in CI environment'],
+    ])
+    t.strictSame(OUTPUT, [], 'no message about interactive shell')
+  })
+
+  t.end()
+})
+
 t.test('npm exec foo, not present locally or in central loc', async t => {
   const path = t.testdir()
   const installDir = resolve('cache-dir/_npx/f7fbba6e0636f890')
